require 'spec_helper'
require 'rake'

describe Rake do
  subject(:run_task) { dc_task['armaf:run'].execute }

  let(:this_file) { File.expand_path(__FILE__) }
  let(:dc) { described_class }
  let(:dc_task) { dc::Task }

  let(:app) do
    module TestingThis
    end

    TestingThis
  end

  let(:user_model) do
    next app::User if defined?(app::User)

    app.const_set(:User, Class.new do
      attr_accessor :code_file, :id
      def save
        true
      end

      def self.ancestors
        [ActiveRecord::Base, *super]
      end
    end)
  end

  let(:user_one) { user_model.new }
  let(:user_two) { user_model.new.tap { |u| u.code_file = 123 } }
  let(:users_list) { [user_one, user_two] }

  before do
    allow(ArmaFixer::Execute).to receive(:show).with(any_args).and_return(nil)

    expect(user_one).to receive(:save)
    expect(user_two).to_not receive(:save)

    dc.load_rakefile('Rakefile')
    dc_task.define_task(:environment)
    allow(user_model).to receive(:where).with('').and_return(users_list)

    users_list.class_eval do
      alias_method :find_each, :each
    end

    stub_const('ArmaFixer::APP', app)
    stub_const('ArmaFixer::MODELS_HASH', models_hash)
  end

  describe 'rake armaf:run' do
    after do
      expect(user_one.code_file).to be_a File
      expect(user_two.code_file).to eq 123
    end

    context 'MODELS_HASH is an array' do
      before { stub_const('ArmaFixer::DEFAULT_IMAGE_PATH', this_file) }
      let(:models_hash) { { user_model.to_s => [:code_file] } }
      it { run_task }
    end

    context 'MODELS_HASH is a hash' do
      let(:models_hash) { { user_model.to_s => { code_file: this_file } } }
      it { run_task }
    end
  end
end
